@@ -2,10 +2,11 @@ package URI::Namespace;
 use Moo 1.006000;
 use URI;
 use IRI 0.003;
+use Scalar::Util qw(blessed);
 use Types::Namespace 0.004 qw( Iri );
 use namespace::autoclean;
 
-our $VERSION = '1.02';
+our $VERSION = '1.08';
 
 =head1 NAME
 
@@ -44,6 +45,13 @@ Returns a L<IRI> object with the namespace IRI. Optionally, the method
 can take a local part as argument, in which case, it will return the
 namespace IRI with the local part appended.
 
+=item C<< local_part ( $uri ) >>
+
+Returns the local part string if the given argument URI (which may be
+a string, L<URI> or L<IRI> object) matches the namespace URI, or
+C<undef> if not.
+
+
 =back
 
 The following methods from L<URI> can be used on an URI::Namespace object: C<as_string>, C<as_iri>, C<canonical>, C<eq>, C<abs>, C<rel>.
@@ -83,8 +91,8 @@ sub iri {
 	my ($self, $name) = @_;
 	if (defined($name)) {
 		my $str = $self->_uri->as_string;
-		my $lastc = substr($str, -1); # Find the last character of the string
-		$str .= '#' unless (($lastc eq '#') or ($lastc eq '/'));
+		# XSD is given without hash in XML world, but hash should be added anyway (see issue #14)
+		$str .= '#' if ($str eq 'http://www.w3.org/2001/XMLSchema');
 		return IRI->new($str . "$name");
 	} else {
 		return $self->_uri;
@@ -95,14 +103,29 @@ sub uri {
 	my ($self, $name) = @_;
 	my $iri = $self->_uri->as_string;
 	if (defined($name)) {
-		my $lastc = substr($iri, -1); # Find the last character of the string
-		$iri .= '#' unless (($lastc eq '#') or ($lastc eq '/')); 
+		$iri .= '#' if ($iri eq 'http://www.w3.org/2001/XMLSchema');
 		return URI->new($iri . "$name");
 	} else {
 		return URI->new($iri);
 	}
 }
 
+sub local_part {
+  my ($self, $fulluri) = @_;
+  my $local_part = undef;
+  if (blessed($fulluri) && $fulluri->isa('IRI')) {
+	 $fulluri = $fulluri->as_string;
+  }
+  my $iri = $self->_uri->as_string;
+  if ($fulluri =~ m/^$iri(.+)/) {
+	 $local_part = $1;
+  } else {
+	 # TODO: throw error?
+  }
+  return $local_part;
+}
+
+
 for my $method (qw/ abs rel eq canonical /) {
 	eval qq[ sub $method { shift->uri->${method}(\@_) } ];
 }
