@@ -1,3 +1,16 @@
+=pod
+
+=head1 NAME
+
+URI::NamespaceMap - Class holding a collection of namespaces
+
+=head1 VERSION
+
+Version 1.08
+
+=cut
+
+
 package URI::NamespaceMap;
 use Moo 1.006000;
 use Module::Load::Conditional qw[can_load];
@@ -11,17 +24,8 @@ use Types::Namespace 0.004 qw(Namespace);
 use URI::NamespaceMap::ReservedLocalParts;
 use namespace::autoclean;
 
-=head1 NAME
 
-URI::NamespaceMap - Class holding a collection of namespaces
-
-=head1 VERSION
-
-Version 1.04
-
-=cut
-
-our $VERSION = '1.04';
+our $VERSION = '1.08';
 
 
 =head1 SYNOPSIS
@@ -34,6 +38,10 @@ our $VERSION = '1.04';
   $map->add_mapping(rdf => 'http://www.w3.org/1999/02/22-rdf-syntax-ns#' );
   $map->list_prefixes;  #  ( 'foaf', 'rdf', 'xsd' )
   $map->foaf; # Returns URI::Namespace object
+  while (my ($prefix, $nsURI) = $map->each_map) {
+	 $node->setNamespace($nsURI->as_string, $prefix); # For use with XML::LibXML
+  }
+
 
 =head1 DESCRIPTION
 
@@ -47,11 +55,18 @@ This module provides an object to manage multiple namespaces for creating L<URI:
 
 Returns a new namespace map object. You can pass a hash reference with
 mappings from local names to namespace URIs (given as string or
-L<RDF::Trine::Node::Resource>) or namespaces_map with a hashref. You
-may also pass an arrayref containing just prefixes and/or namespace
-URIs, and the module will try to guess the missing part. To use this
-feature, you need L<RDF::NS::Curated>, L<RDF::NS>, L<XML::CommonNS> or
-L<RDF::Prefixes>, or preferably all of them.
+L<RDF::Trine::Node::Resource>) or namespaces_map with a hashref. 
+
+You may also pass an arrayref containing just prefixes and/or
+namespace URIs, and the module will try to guess the missing part. To
+use this feature, you need L<RDF::NS::Curated>, L<RDF::NS>,
+L<XML::CommonNS> or L<RDF::Prefixes>, or preferably all of them. With
+that, you can do e.g.
+
+  my $map = URI::NamespaceMap->new( 'rdf', 'xsd', 'foaf' );
+
+and have the correct mappings added automatically.
+
 
 
 =item C<< add_mapping ( $name => $uri ) >>
@@ -88,6 +103,11 @@ Returns an array of L<URI::Namespace> objects with all the namespaces.
 
 Returns an array of prefixes.
 
+=item C<< each_map >>
+
+Returns an 2-element list where the first element is a prefix and the
+second is the corresponding L<URI::Namespace> object.
+
 =cut
 
 around BUILDARGS => sub {
@@ -126,6 +146,7 @@ sub remove_mapping  { delete $_[0]->namespace_map->{$_[1]} }
 sub namespace_uri   { $_[0]->namespace_map->{$_[1]} }
 sub list_namespaces { values %{ $_[0]->namespace_map } }
 sub list_prefixes   { keys   %{ $_[0]->namespace_map } }
+sub each_map        { each   %{ $_[0]->namespace_map } }
 
 
 =item C<< guess_and_add ( @string_or_uri ) >>
@@ -184,6 +205,9 @@ sub _scrub_uri {
 			if ($uri->isa('URI::Namespace')) {
 				$uri = $uri->as_string;
 			}
+			elsif ($uri->isa('IRI')) {
+				$uri = $uri->as_string;
+			}
 			elsif ($uri->isa('URI')) {
 				# it's probably not necessary to do this, but whatever
 				$uri = $uri->as_string;
@@ -196,6 +220,9 @@ sub _scrub_uri {
 				# and this
 				$uri = $uri->uri->uri_value;
 			}
+#			elsif ($uri =~ m/^\<(.*?)\>$/) {
+#			  $uri = $1;
+#			}
 			else {
 				# let's hope whatever was passed in has a string overload
 				$uri = "$uri";
@@ -257,7 +284,6 @@ sub abbreviate {
 
 	my $prefix = $self->prefix_for($uri);
 
-	# XXX is this actually the most desirable behaviour?
 	return unless defined $prefix;
 
 	my $nsuri = _scrub_uri($self->namespace_uri($prefix));
@@ -313,7 +339,7 @@ sub _guess {
 				my $ns = RDF::NS->new;
 				$namespaces{$prefix} = $ns->SELECT($entry);
 			}
-			carp "Cannot resolve '$entry'" unless $namespaces{$prefix};
+			carp "Cannot resolve assumed prefix '$entry'" unless $namespaces{$prefix};
 		} else {
 			# Lets assume a URI string
 			carp "Cannot resolve '$entry' without RDF::NS::Curated, RDF::NS or RDF::Prefixes" unless ($rnscu || $rdfns || $rdfpr);
@@ -331,7 +357,7 @@ sub _guess {
 				$prefix = $context->get_prefix($entry);
 			}
 			unless ($prefix) {
-				carp "Cannot resolve '$entry'";
+				carp "Cannot resolve assumed URI string '$entry'";
 			} else {
 				my $i = 1;
 				while ($r->is_reserved($prefix)) {
@@ -382,7 +408,7 @@ You can find documentation for this module with the perldoc command.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2012,2013,2014,2015,2016,2017 Gregory Todd Williams, Chris Prather and Kjetil Kjernsmo
+Copyright 2012,2013,2014,2015,2016,2017,2018 Gregory Todd Williams, Chris Prather and Kjetil Kjernsmo
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
